package ru.dbotthepony.mc.otm.triggers

import com.google.gson.JsonObject
import com.mojang.serialization.codecs.RecordCodecBuilder
import net.minecraft.advancements.Criterion
import net.minecraft.advancements.critereon.*
import net.minecraft.advancements.critereon.MinMaxBounds.Doubles
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerPlayer
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.data.Codec2TriggerSerializer
import java.util.Optional

object NanobotsArmorTrigger : SimpleCriterionTrigger<NanobotsArmorTrigger.Instance>() {
	val id = ResourceLocation(OverdriveThatMatters.MOD_ID, "nanobots_armor")

	override fun createInstance(
		p_66248_: JsonObject,
		p_286603_: Optional<ContextAwarePredicate>,
		p_66250_: DeserializationContext
	): Instance {
		return codec.fromJson(p_66248_, p_286603_, p_66250_)
	}

	val codec = Codec2TriggerSerializer<Instance> { p ->
		RecordCodecBuilder.create {
			it.group(
				Doubles.CODEC.fieldOf("predicate").forGetter(Instance::predicate),
				p.playerPredicate.forGetter { it.playerPredicate() },
			).apply(it, ::Instance)
		}
	}

	fun trigger(player: ServerPlayer, damageAbsorbed: Double) {
		trigger(player) { it.predicate.matches(damageAbsorbed) }
	}

	class Instance(val predicate: Doubles, player: Optional<ContextAwarePredicate> = Optional.empty()) : AbstractCriterionTriggerInstance(player) {
		override fun serializeToJson(): JsonObject {
			return codec.toJson((this))
		}

		fun criterion() = Criterion(NanobotsArmorTrigger, this)
	}
}
